/**
 *  The ABSTree class serves as the superclass for a given tree 
 *  implementation. It's sole purpose is to serve as a generic
 *  interface for use with a tree simulation program.
 * 
 *  <p> As denoted by its name this Tree class is to provide an array based
 *     representation of a tree to the user program. Specifically, the
 *    data in the tree should be represented in the array as follows </p>
 * <br> For example a binary tree would be represented as follows in an array
 * <pre> *NOTE: the underscore ( _ )  represents a null or blank array spot
 *      given a Tree with nodes      
 *             R
 *         P      T
 *      M    Q  S   Z
 * would be represented in the array as
 * [ _ R P T M Q S Z ]
 * </pre>
 *
 * <p> A similarly built tree with missing items would be represented as
 *   follows </p>
 *<pre> 
 * If you are given a Tree with nodes  {A,B,D,G,I,J,K,M}    
 *            G
 *         D      K
 *      A   _   I   M
 *     _ B _ _ _ J _ _
 * should be represented in the array as
 * [ _ G D K A _ I M _ B _ _ _ J _ _ ]
 * </pre>
 *
 * <i> NOTE: The first item is to remain null or unused. Doing so 
 *  allows easier tree manipulation; for example the Left Child
 *  for Subtree_x  where x is the nth node in the array position is located
 *  at the array position =  x * 2 </i>
 *
 *
 * @author Kennard Love
 */

public abstract class ABSTree {

    /**
	 *  The tree's data Array.  Element at index 0 is skipped.
	 *  Left child is parent's index * 2 and the right child is
	 *  the parent's index * 2 + 1
	 */
    protected String[] treeData;  

    /**
     *  This function simply supplies the representative tree data..
     *  @return the Array of strings represented in the tree.
     */
    public String[] getArray(){return treeData;};
    
	/**
	 * Function display's the tree based on the treeData array.
	 * For instance, if the data array contained the strings:
	 *<p><pre>
	 *{"Root", "L","R", "LL","LR", "RL","RR", "","","LRL","", "RLL",""}
	 *</pre></p>
	 *<p> The tree would be displayed as <pre>
	 *Root
	 *L R
	 *LL LR RL RR
	 *_ _ LRL _ RLL _ _ _
	 *
	 * *NOTE: _ will actually print to the console.
	 *</pre></p>
	 * The output of the tree will not exceed the height of the tree returned
	 * by getHeight.
	 *
	 */
	public void printTree(){
		int count = 1;
		System.out.println("Tree Height: "+getHeight());
		if((treeData[count]==null)){
			System.out.println("TREE EMPTY");
			return;
		}

		for(int cur_level=0; count<treeData.length 
				&& count<=Math.pow(2,getHeight()+1);
			cur_level++){
		
			for(int inner=1; 
				inner<=Math.pow(2,cur_level) && count<treeData.length;
				inner++){
				if(treeData[count] == null || treeData[count].compareTo("")==0)
					System.out.print("_ ");
				else
					System.out.print(treeData[count]+" ");
				count++;
			}
			System.out.println("");
		}
	}

    /**
     * Allows a new data item to be inserted into the tree
     * according to the specific tree implementation.
     * <p><i> You Must implement this yourself.</i></p>
     *
     * @param item the data value to insert into the tree
     */
    abstract public void insertItem(String item);

    /**
     * This function should return the current number of items in the tree.
     * @return value returned is the number of items currently in the tree.
     */
    abstract public int getItemCount();

    /**
     * This method returns the height of the tree.
	 * It should look through the tree and calculate the depth
     * of the lowest leaf in the tree and return that value.
	 * <p>The height of a one node tree is one.</p>
     *
     * @return the depth of the lowest leaf node in the tree
     */
    abstract public int getHeight();

	/**
	 * The funciton performs a binary search for the search term.
	 *
	 * @param search_term the string target value
	 * @return returns the string if found or null if not found
	 */
	abstract public String search(String search_term);

	/**
	 * Deletes the item identified by the parameter. In doing so, this
	 * function maintain the order, location and/or balance of the tree's
	 * nodes (if necessary).
	 * <p> if the delete operation fails for any reason the function
	 * should return null </p>
	 *
	 * @param item_to_delete The string value of the item to delete
	 * @return the return value is either the value of the string deleted
	 *         or <i>null</i> if the item could not be deleted.
	 *
	 */
	 abstract public String deleteItem(String item_to_delete);

	/**
	 * Traversal function returns the representation of the tree
	 * based on an in-order tree traversal: visit in order Left-Root-Right
	 *
	 * @return a string representation of the in-order traversal
	 */
	abstract public String inorderTraversal();

	/**
	 * Traversal function returns the representation of the tree
	 * based on an in-order tree traversal: visit in order Root-Left-Right
	 *
	 * @return the string array representation of the in-order traversal
	 *
	 */
	abstract public String preorderTraversal();

	/**
	 * Traversal function returns the representation of the tree
	 * based on an in-order tree traversal: visit in order Left-Right-Root
	 *
	 * @return the string array representation of the in-order traversal
	 *
	 */
	abstract public String postorderTraversal();
}
